/*
    KWin - the KDE window manager
    This file is part of the KDE project.

    SPDX-FileCopyrightText: 2017 Martin Gräßlin <mgraesslin@kde.org>
    SPDX-FileCopyrightText: 2022 KylinSoft Co., Ltd.

    SPDX-License-Identifier: GPL-2.0-or-later
*/
"use strict";
var appearDesktop;
var disappearDesktop;
var desktopName = '';
var currentTabletMode = false;
var scaleAppearDesktopAnimateId;
var scaledesktopEffect = {
    
    duration: animationTime(250),
    loadConfig: function () 
    {
        scaledesktopEffect.duration = animationTime(250);
    },

    setNonDesktopWindowsTransparency: function()
    {
        var stackingOrder = effects.stackingOrder;
        var w;
        for (var i = 0; i < stackingOrder.length; ++i) {
            w = stackingOrder[i];
            if (!w.visible || w.dock || w.caption == "" || w.caption == "tablet_blur_background — UKUI-KWin"
                    || w.slurpedByEyeOnScreen || w.caption == "UKUI Panel") {
                continue;
            }

            if(!w.desktopWindow) {
                animate({
                    window: w,
                    animations: [{
                        type: Effect.Opacity,
                        curve: QEasingCurve.Linear,
                        duration: 3 * scaledesktopEffect.duration,
                        from: 0,
                        to: 0
                    }]
                });
            }
        }
    },

    appearAnimate: function ()
    {
        if (typeof(appearDesktop) == "undefined") {
            return;
        }
        cancel(scaleAppearDesktopAnimateId);
        delete scaleAppearDesktopAnimateId;
        
        scaledesktopEffect.setNonDesktopWindowsTransparency();
        var windowRect = appearDesktop.geometry;
        animate({
            window: appearDesktop,
            duration: 2 * scaledesktopEffect.duration, 
            animations: [
            {
                type: Effect.Size,
                from: {
                    value1: windowRect.width * 0.9,
                    value2: windowRect.height * 0.9
                },
                to: {
                    value1: windowRect.width,
                    value2: windowRect.height
                }
            },
            {
                type: Effect.Opacity,
                from: 0,
                to: 1.0
            }
        ]
        }); 
    },

    disappearAnimate: function ()
    {
        var windowRect = disappearDesktop.geometry;

        animate({
            window: disappearDesktop,
            duration: 2 * scaledesktopEffect.duration, 
            
            animations: [
                {
                    type: Effect.Size,
                    from: {
                        value1: windowRect.width,
                        value2: windowRect.height
                    },
                    to: {
                        value1: windowRect.width * 0.9,
                        value2: windowRect.height * 0.9
                    }
                },
                {
                    type: Effect.Opacity,
                    from: 1.0,
                    to: 0.0
                }
            ]
        });
    },

    scaleAppearDesktopAnimate: function ()
    {
        var windowRect = disappearDesktop.geometry;

        scaleAppearDesktopAnimateId = set({
            window: appearDesktop,
            duration: 2 * scaledesktopEffect.duration, 
            
            animations: [
                {
                    type: Effect.Size,
                    from: {
                        value1: windowRect.width,
                        value2: windowRect.height
                    },
                    to: {
                        value1: windowRect.width * 0.9,
                        value2: windowRect.height * 0.9
                    }
                },
                {
                    type: Effect.Opacity,
                    from: 1.0,
                    to: 0.0
                }
            ]
        });
    },

    setDesktopDisplayStatus: function(win)
    { 
        if(win.desktopWindow) { 
            if (desktopName == win.caption 
                || (currentTabletMode && win.caption == "ukui-tablet-desktop")
                || (!currentTabletMode && win.caption != "ukui-tablet-desktop")) {
                return;
            }
            desktopName = win.caption;
            effects.setDesktopHide();
        }
    },

    setDisappearDesktop : function (tabletmode, pcDesktop, tmDesktop) {
        disappearDesktop = tabletmode ? pcDesktop : tmDesktop;
    },

    setAppearDesktop : function (tabletmode, pcDesktop, tmDesktop) {
        appearDesktop = tabletmode ? tmDesktop : pcDesktop;
    },

    animationControl : function () {
        if(typeof(disappearDesktop) != "undefined" && typeof(appearDesktop) != "undefined") { 
            scaledesktopEffect.disappearAnimate();
            scaledesktopEffect.scaleAppearDesktopAnimate();
        } else if (typeof(disappearDesktop) != "undefined" && typeof(appearDesktop) == "undefined") {
            effects.setDesktopHide();
            scaledesktopEffect.disappearAnimate();
        } else if (typeof(disappearDesktop) == "undefined" && typeof(appearDesktop) != "undefined") {
            effects.setDesktopHide();
            scaledesktopEffect.appearAnimate();
        } else {
            effects.setDesktopHide();
        }
    },

    filterNonDesktopWindows : function (tabletmode) 
    {
        var pcDesktop;
        var tmDesktop;
        currentTabletMode = tabletmode;
        var stackingOrder = effects.stackingOrder;
        var w;
        for (var i = 0; i < stackingOrder.length; ++i) {
            w = stackingOrder[i];

            if (!w.visible || w.dock || w.caption == "" || w.caption == "tablet_blur_background — UKUI-KWin" 
            || w.slurpedByEyeOnScreen || w.caption == "UKUI Panel") {
                continue;
            }

            if(w.desktopWindow) {
                if(w.caption == "ukui-tablet-desktop") {
                    tmDesktop = w;
                } else {
                    pcDesktop = w;
                }
            } else {
                animate({
                    window: w,
                    animations: [{
                        type: Effect.Opacity,
                        curve: QEasingCurve.Linear,
                        duration: 2 * scaledesktopEffect.duration,
                        to: 0
                    }]
                });
            }
        }
        scaledesktopEffect.setDisappearDesktop(tabletmode, pcDesktop, tmDesktop);
        scaledesktopEffect.setAppearDesktop(tabletmode, pcDesktop, tmDesktop);
        scaledesktopEffect.animationControl();
    },

    init: function () {
        scaledesktopEffect.loadConfig();
        effects.tabletModeChanged.connect(scaledesktopEffect.filterNonDesktopWindows);
        effects.desktopAppearAnimatestart.connect(scaledesktopEffect.appearAnimate)
        effect.animationEnded.connect(scaledesktopEffect.setDesktopDisplayStatus);
    }
};

scaledesktopEffect.init();
